#!/usr/bin/env python
"""
applet.py
by Jason Conti
February 16, 2010

The main applet code.
"""

import pygtk
pygtk.require('2.0')

import gtk
import gnomeapplet
import gobject
import logging
import os.path
import pango
import sys

from optparse import OptionParser

from Notifications import Notifications

class Applet(object):
  """The main applet wrapper."""
  def __init__(self, applet, iid):
    self._applet = applet

    self._unread_messages = 0

    logging.debug("Configuring Button")
    self._button = gtk.ToggleButton("No Unread Messages")
    self._button.connect("button_press_event", self.do_not_eat_button_press)
    self._button.connect("toggled", self.on_button_toggled)
    self._applet.add(self._button)

    logging.debug("Configuring Menu")
    self._applet.setup_menu("""
      <popup name="button3">
        <menuitem name="About" verb="About" _label="_About" pixtype="stock" pixname="gtk-about"/>
      </popup>
      """, [
      ("About", lambda a, b: self.on_show_about(a))
      ])

    logging.debug("Creating message window")
    self._window = MessageWindow()

    logging.debug("Showing applet")
    self._applet.connect("destroy", self.on_applet_destroyed)
    self._applet.show_all()

    logging.debug("Starting notifier")
    self._notify = Notifications(os.path.expanduser("~/.cache/notify-osd.log"))
    self._notify.add_listener(self.on_messages_received)
    self._notify.start()

  def get_orient(self):
    return self._applet.get_orient()

  def get_origin(self):
    return self._button.get_window().get_origin()

  def get_screen(self):
    return self._button.get_window().get_screen()

  def get_size(self):
    return self._button.get_window().get_size()

  def update_unread_messages(self):
    if self._unread_messages == 0:
      label = "No Unread Messages"
    elif self._unread_messages == 1:
      label = "1 Unread Message"
    else:
      label = str(self._unread_messages) + " Unread Messages"

    self._button.set_label(label)

  # Callbacks

  def do_not_eat_button_press(self, widget, event):
    """Adapted from the GNOME clock applet."""
    if event.button != 1:
      widget.stop_emission("button_press_event")

    return False

  def on_applet_destroyed(self, *args):
    self._notify.stop()

  def on_button_toggled(self, button, *args):
    if button.get_active():
      self._window.show_all()
      self._window.align_with_applet(self)
    else:
      self._window.hide_all()
      self._unread_messages = 0
      self.update_unread_messages()

  def on_messages_received(self, messages):
    self._window.add_messages(messages)
    self._unread_messages += len(messages)
    self.update_unread_messages()

  def on_show_about(self, *args):
    about = gtk.AboutDialog()

    about.set_authors(["Jason Conti <jason.conti@gmail.com>"])
    about.set_comments("View recent notifications")
    about.set_copyright(u"Copyright \xa9 2010 Jason Conti.")
    about.set_name("Recent Notifications")
    about.set_version("1.0.0")

    about.connect("response", lambda self, *args: self.destroy())
    about.set_screen(self.get_screen())
    about.show_all()

class MessageWindow(gtk.Window):
  """Shows the log of notification messages."""
  def __init__(self, limit_messages = 5):
    gtk.Window.__init__(self)

    self._limit_messages = limit_messages

    self.set_type_hint(gtk.gdk.WINDOW_TYPE_HINT_DOCK)
    self.set_decorated(False)
    self.set_resizable(False)
    self.stick()
    self.set_title("Message Window")
    self.set_border_width(5)
    self.set_size_request(400, -1)

    self._vbox = gtk.VBox(True, 5)

    self._messages = []

    self.add(self._vbox)

  def add_messages(self, messages):
    """Adds many messages at once. Since it limits the number of messages
    displayed, will only add the last n messages, where n = limit_messages."""
    m = messages
    if len(m) > self._limit_messages:
      m = m[len(m) - self._limit_messages:]

    for message in m:
      self.add_message(message)

  def add_message(self, message):
    """Adds a message to the message window."""
    widget = MessageWidget(message)
    self._messages.append(widget)
    self._vbox.pack_start(widget)

    if len(self._messages) > self._limit_messages:
      message = self._messages.pop(0)
      self._vbox.remove(message)
      message.destroy()

  def align_with_applet(self, applet):
    """Adapted from position_calendar_popup in the GNOME clock applet."""
    # Get the size of the applet button and the size of this window
    x, y = applet.get_origin()
    w, h = self.size_request()
    button_w, button_h = applet.get_size()

    screen = self.get_screen()
    monitor = None

    # Figure out which monitor we are on
    for i in range(screen.get_n_monitors()):
      m = screen.get_monitor_geometry(i)

      if x >= m.x and x <= m.x + m.width and y >= m.y and y <= m.y + m.height:
        monitor = m
        break

    # Probably xinerama
    if monitor == None:
      monitor = gtk.gdk.Rectangle(0, 0, screen.get_width(), screen.get_height())

    orient = applet.get_orient()

    # Figure out the x, y coordinates of the window and the gravity depending
    # on the orientation of the panel
    if orient == gnomeapplet.ORIENT_RIGHT:  # Panel on left
      x += button_w
      if (y + h) > monitor.y + monitor.height:
        y -= (y + h) - (monitor.y + monitor.height)

      if (y + h) > (monitor.height / 2):
        gravity = gtk.gdk.GRAVITY_SOUTH_WEST
      else:
        gravity = gtk.gdk.GRAVITY_NORTH_WEST

    elif orient == gnomeapplet.ORIENT_LEFT: # Panel on right
      x -= w
      if (y + h) > monitor.y + monitor.height:
        y -= (y + h) - (monitor.y + monitor.height)

      if (y + h) > (monitor.height / 2):
        gravity = gtk.gdk.GRAVITY_SOUTH_EAST
      else:
        gravity = gtk.gdk.GRAVITY_NORTH_EAST

    elif orient == gnomeapplet.ORIENT_DOWN: # Panel on top
      y += button_h
      if (x + w) > monitor.x + monitor.width:
        x -= (x + w) - (monitor.x + monitor.width)

      gravity = gtk.gdk.GRAVITY_NORTH_WEST

    else: # Panel on bottom
      y -= h
      if (x + w) > monitor.x + monitor.width:
        x -= (x + w) - (monitor.x + monitor.width)

      gravity = gtk.gdk.GRAVITY_SOUTH_WEST

    self.move(x, y)
    self.set_gravity(gravity)

class MessageWidget(gtk.Frame):
  """Widget to display a notifications.Message."""
  def __init__(self, message):
    gtk.Frame.__init__(self)

    self._message = message

    self._vbox = gtk.VBox(False, 5)
    self._vbox.set_border_width(5)
    self.add(self._vbox)

    self._title_label = gtk.Label(message.title)
    attr = pango.AttrList()
    attr.insert(pango.AttrWeight(pango.WEIGHT_BOLD, 0, -1))
    self._title_label.set_attributes(attr)
    self._vbox.pack_start(self._title_label)

    self._body_label = gtk.Label(message.body)
    self._body_label.set_line_wrap(True)
    self._vbox.pack_start(self._body_label)

    self._timestamp_label = gtk.Label(message.formatted_time() + " from " + message.process_name)
    attr = pango.AttrList()
    attr.insert(pango.AttrScale(pango.SCALE_SMALL, 0, -1))
    self._timestamp_label.set_attributes(attr)
    self._vbox.pack_start(self._timestamp_label)

def applet_factory(applet, iid):
  Applet(applet, iid)
  return True

def run_in_window():
  main_window = gtk.Window()
  main_window.set_title("Applet")
  main_window.connect("destroy", gtk.main_quit)
  app = gnomeapplet.Applet()
  applet_factory(app, None)
  app.reparent(main_window)
  main_window.show_all()
  gtk.main()

def initialize_applet():
  gnomeapplet.bonobo_factory("OAFIID:RecentNotificationsApplet_Factory", 
      gnomeapplet.Applet.__gtype__, 
      "hello", "0", applet_factory)

# Get command line options
def parse_options(args):
  usage = "Usage: %prog [options]"
  parser = OptionParser(usage=usage)

  parser.add_option("-w", "--windowed", action="store_true", dest="windowed", default=False, metavar=" ",
      help="run the applet in a window for testing purposes [default: %default]")
  # Need to specify these options, because they are provided when adding the applet to a panel
  # and OptionParser raises an exception for invalid args
  parser.add_option("--oaf-activate-iid")
  parser.add_option("--oaf-ior-fd")

  options, extra = parser.parse_args(args)

  return options

def main():
  logging.basicConfig(filename=os.path.expanduser("~/.cache/recent-notifications.log"),
      level=logging.DEBUG)

  logging.debug("Parsing options")
  options = parse_options(sys.argv)

  if options.windowed:
    logging.debug("Running windowed")
    run_in_window()
  else:
    logging.debug("Initializing Applet")
    initialize_applet()

if __name__ == '__main__':
  main()
